/*
 * fancydbdriver.h
 *
 *  Created on: 28.11.2009
 *      Author: wglas
 */

#ifndef FANCYDBDRIVER_H_
#define FANCYDBDRIVER_H_

#include <fancydb.h>
#include <fancyobject.h>

#define FANCYDB_DRIVER_ABI_VERSION "1"

namespace Fancy {

class Map;

namespace DB {

class Connection;
class Driver;

typedef Fancy::ObjPtr<Driver> (*new_driver_instance_func_t)();

/**
 * A database driver.
 */
class FANCYDB_API Driver: public virtual Fancy::Object {

public:
	typedef Fancy::ObjPtr<Driver> ptr_t;

	/**
	 * Get the static instance of a driver. If the driver has not been
	 * instantiated before, it will be lazily created.
	 * @param name The name of the driver.
	 * @return The shared, static instance of this driver.
	 */
	static ptr_t getInstance(const char *name);

	/**
	 * Register a user-implemented driver.
	 *
	 * @param name The name of the driver.
	 * @param ctor The constructor function.
	 */
	static void registerDriver(const char *name, new_driver_instance_func_t ctor);

	/**
	 * Open a connection to the database. This method is equivalent to calling
     * the options-aware method with a null options argument.
	 * @param url The url to connect to. The scheme of the URL is a driver name.
	 * @param user The database user to connect.
	 * @param password The password used for the connection.
	 * @return A new, open connection.
	 */
	virtual Fancy::ObjPtr<Connection> connect(const std::string &url, const std::string &user, const std::string &password) = 0;

	/**
	 * Open a connection to the database.
	 * @param url The url to connect to. The scheme of the URL is a driver name.
	 * @param user The database user to connect.
	 * @param password The password used for the connection.
	 * @param options Database-sepcific options with strings as keys and
     *                option-specific objects such as Boolean, Integer or
     *                String as values. Refer to your drivers documentation for supported
     *                options.
	 * @return A new, open connection.
	 */
	virtual Fancy::ObjPtr<Connection> connect(const std::string &url, const std::string &user, const std::string &password, const Fancy::ObjPtr<Fancy::Map> &options) = 0;

	/**
	 * Open a connection to a database. The individual driver is
	 * fetched from the scheme of the given URL. This method is equivalent to calling
     * the options-aware method with a null options argument.
	 * @param url The url to connect to. The scheme of the URL is a driver name.
	 * @param user The database user to connect.
	 * @param password The password used for the connection.
	 * @return A new, open connection.
	 */
	static Fancy::ObjPtr<Connection> connect_s(const std::string &url, const std::string &user, const std::string &password);

	/**
	 * Open a connection to a database. The individual driver is
	 * fetched from the scheme of the given URL.
	 * @param url The url to connect to. The scheme of the URL is a driver name.
	 * @param user The database user to connect.
	 * @param password The password used for the connection.
	 * @param options Database-sepcific options with strings as keys and
     *                option-specific objects such as Boolean, Integer or
     *                String as values. Refer to your drivers documentation for supported
     *                options.
	 * @return A new, open connection.
	 */
	static Fancy::ObjPtr<Connection> connect_s(const std::string &url, const std::string &user, const std::string &password, const Fancy::ObjPtr<Fancy::Map> &options);

protected:
	Driver();
	virtual ~Driver();
};

}

}

#endif /* FANCYDBDRIVER_H_ */
